@@ -3,7 +3,6 @@ use strict;
 use Exporter 'import';
 use JSON;
 use Carp qw(croak cluck);
-use MozRepl;
 use Scalar::Util qw(refaddr weaken);
 
 =head1 NAME
@@ -40,7 +39,7 @@ MozRepl::RemoteObject - treat Javascript objects as Perl objects
 =cut
 
 use vars qw[$VERSION $objBridge @CARP_NOT @EXPORT_OK $WARN_ON_LEAKS];
-$VERSION = '0.21';
+$VERSION = '0.22';
 
 @EXPORT_OK=qw[as_list];
 @CARP_NOT = (qw[MozRepl::RemoteObject::Instance
@@ -332,9 +331,16 @@ C<launch> option:
 
 =cut
 
+sub require_module($) {
+    local $_ = shift;
+    s{::|'}{/}g;
+    require "$_.pm"; # dies if the file is not found
+};
+
 sub install_bridge {
     my ($package, %options) = @_;
     $options{ repl } ||= $ENV{MOZREPL};
+    my $repl_class = delete $options{ repl_class } || $ENV{MOZREPL_CLASS} || 'MozRepl';
     $options{ constants } ||= {};
     $options{ log } ||= [qw/ error/];
     $options{ queue } ||= [];
@@ -352,7 +358,8 @@ sub install_bridge {
             push @host_port, port => $2
                 if defined $2;
         };
-        $options{repl} = MozRepl->new();
+        require_module $repl_class;
+        $options{repl} = $repl_class->new();
         RETRY: {
             my $ok = eval {
                 $options{repl}->setup({
@@ -516,6 +523,7 @@ sub queued {
         #my $js = join "//\n;//\n", @{ $self->queue };
         my $js = join "\n", map { /;$/? $_ : "$_;" } @{ $self->queue };
         # we don't want a result here!
+        # This is where we would do ->execute_async on AnyEvent
         $self->repl->execute($js);
         @{ $self->queue } = ();
     };
@@ -669,6 +677,7 @@ sub js_call_to_perl_struct {
         #warn "Returning result of $js";
         $js = "${queue_pre}JSON.stringify( function(){ var res = $js; return { result: res }}())$queue_post";
         #warn $js;
+        # When going async, we would want to turn this into a callback
         my $res = $repl->execute($js);
         $res =~ s/^(?:\.+\>\s+)+//g;
         while ($res !~ /\S/) {
@@ -681,6 +690,7 @@ sub js_call_to_perl_struct {
         return $d->{result}
     } else {
         #warn "Executing $js";
+        # When going async, we would want to turn this into a callback
         $repl->execute($js);
         ()
     };
@@ -771,8 +781,10 @@ package # hide from CPAN
 use strict;
 use Carp qw(croak cluck);
 use Scalar::Util qw(blessed refaddr);
+use MozRepl::RemoteObject::Methods;
 push @Carp::CARP_NOT, __PACKAGE__;
 
+# Move to ::methods
 use overload '%{}' => '__as_hash',
              '@{}' => '__as_array',
              '&{}' => '__as_code',
@@ -850,7 +862,8 @@ sub AUTOLOAD {
     my $fn = $MozRepl::RemoteObject::Instance::AUTOLOAD;
     $fn =~ s/.*:://;
     my $self = shift;
-    return $self->__invoke($fn,@_)
+    #return $self->__invoke($fn,@_)
+    return $self->MozRepl::RemoteObject::Methods::invoke($fn,@_)
 }
 
 =head1 EVENTS / CALLBACKS
@@ -913,21 +926,7 @@ by this package:
 
 =cut
 
-sub __invoke {
-    my ($self,$fn,@args) = @_;
-    my $id = $self->__id;
-    die unless $self->__id;
-    
-    ($fn) = $self->__transform_arguments($fn);
-    my $rn = $self->bridge->name;
-    @args = $self->__transform_arguments(@args);
-    #use Data::Dumper;
-    local $" = ',';
-    my $js = <<JS;
-$rn.callMethod($id,$fn,[@args])
-JS
-    return $self->bridge->unjson($js);
-}
+*__invoke = \&MozRepl::RemoteObject::Methods::invoke;
 
 =head2 C<< $obj->__transform_arguments(@args) >>
 
@@ -946,29 +945,7 @@ as a number through to Javascript, or to pass digits as a Javascript string.
 
 =cut
  
-sub __transform_arguments {
-    my $self = shift;
-    my $json = $self->bridge->json;
-    map {
-        if (! defined) {
-             'null'
-        } elsif (/^(?:[1-9][0-9]*|0+)$/) {
-            $_
-        } elsif (ref and blessed $_ and $_->isa(__PACKAGE__)) {
-            sprintf "%s.getLink(%d)", $_->bridge->name, $_->__id
-        } elsif (ref and blessed $_ and $_->isa('MozRepl::RemoteObject')) {
-            $_->name
-        } elsif (ref and ref eq 'CODE') { # callback
-            my $cb = $self->bridge->make_callback($_);
-            sprintf "%s.getLink(%d)", $self->bridge->name,
-                                      $cb->__id
-        } elsif (ref) {
-            $json->encode($_);
-        } else {
-            $json->encode($_)
-        }
-    } @_
-};
+*__transform_arguments = \&MozRepl::RemoteObject::Methods::transform_arguments;
 
 =head2 C<< $obj->__id >>
 
@@ -978,13 +955,7 @@ Perl object.
 
 =cut
 
-sub __id {
-    my $class = ref $_[0];
-    bless $_[0], "$class\::HashAccess";
-    my $id = $_[0]->{id};
-    bless $_[0], $class;
-    $id
-};
+*__id = \&MozRepl::RemoteObject::Methods::id;
 
 =head2 C<< $obj->__on_destroy >>
 
@@ -993,17 +964,7 @@ that gets invoked from C<< DESTROY >>.
 
 =cut
 
-sub __on_destroy {
-    my $class = ref $_[0];
-    bless $_[0], "$class\::HashAccess";
-    my $d = $_[0]->{on_destroy};
-    if (@_ == 2) {
-        $_[0]->{on_destroy} = $_[1];
-    };
-    bless $_[0], $class;
-    $d
-};
-
+*__on_destroy = \&MozRepl::RemoteObject::Methods::on_destroy;
 
 =head2 C<< $obj->bridge >>
 
@@ -1013,13 +974,7 @@ Perl object.
 
 =cut
 
-sub bridge {
-    my $class = ref $_[0];
-    bless $_[0], "$class\::HashAccess";
-    my $bridge = $_[0]->{bridge};
-    bless $_[0], $class;
-    $bridge
-};
+*bridge = \&MozRepl::RemoteObject::Methods::bridge;
 
 =head2 C<< $obj->__release_action >>
 
@@ -1220,6 +1175,11 @@ JS
 
 =head2 C<< $obj->__xpath( $query [, $ref ] ) >>
 
+B<DEPRECATED> - this method will vanish in 0.23.
+Use L<MozRepl::RemoteObject::Methods::xpath> instead:
+
+  $obj->MozRepl::RemoteObject::Methods::xpath( $query )
+
 Executes an XPath query and returns the node
 snapshot result as a list.
 
@@ -1325,7 +1285,7 @@ Javascript object C<ID>. You usually do not call this
 directly but use C<< $bridge->link_ids @IDs >>
 to wrap a list of Javascript ids with Perl objects.
 
-The C<onDestroy> parameter should contain a Javascript
+The C<$onDestroy> parameter should contain a Javascript
 string that will be executed when the Perl object is
 released.
 The Javascript string is executed in its own scope
